// ReceiverProxy.cs
//
// PRESTO Starter Kit for Microsoft .NET 3.0
// 2007 Microsoft France
// Published under the CECILL-B Free Software license agreement.
// (http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.txt)
// 
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
// WHETHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE. 
// THE ENTIRE RISK OF USE OR RESULTS IN CONNECTION WITH THE USE OF THIS CODE 
// AND INFORMATION REMAINS WITH THE USER. 
//

using System;
using System.Configuration;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Dispatcher;
using System.ServiceModel.Description;
using System.ServiceModel.Channels;
using System.Runtime.Serialization;

namespace Microsoft.Presto.StarterKit.NonNormative
{
    class ReceiverProxy
    {
        static void Main(string[] args)
        {
            // Create a service host
            Uri baseAddress = new Uri(ConfigurationManager.AppSettings["baseNetPipeAddress"]);
            ServiceHost serviceHost = new ServiceHost(typeof(ReceiverProxyService), baseAddress);

            // Add an endpoint for the INonNormativePresto Interface
            NetNamedPipeBinding binding = new NetNamedPipeBinding();
            binding.Security.Mode = NetNamedPipeSecurityMode.None;
            //NetTcpBinding binding = new NetTcpBinding();
            //binding.Security.Mode = SecurityMode.None;
            //Uri baseAddress = new Uri(ConfigurationManager.AppSettings["baseNetTcpAddress"]);

            EndpointAddress logicalEndpointAddress = new EndpointAddress("urn:PrestoProxyEndpoint");
            ServiceEndpoint serviceEndpoint = new ServiceEndpoint(ContractDescription.GetContract(typeof(INonNormativePresto)), binding, logicalEndpointAddress);
            serviceEndpoint.ListenUriMode = ListenUriMode.Explicit;
            //serviceEndpoint.ListenUri = new Uri(baseAddress + "/" + "proxy");
            serviceEndpoint.ListenUri = new Uri(baseAddress + "/" + "proxy");

            serviceHost.Description.Endpoints.Add(serviceEndpoint);

            // Create and open the service runtime
            serviceHost.Open();

            // Display the Service endpoints state
            Console.WriteLine("Receiver Proxy Service Endpoints:");
            foreach (ChannelDispatcher channelDispatcher in serviceHost.ChannelDispatchers)
            {
                foreach (EndpointDispatcher endpointDispatcher in channelDispatcher.Endpoints)
                {
                    StringBuilder sb = new StringBuilder();
                    sb.AppendFormat("Listener ({0}, {1}) = {2}, State = {3}",
                            endpointDispatcher.ContractName.ToString(),
                            endpointDispatcher.ContractNamespace.ToString(),
                            endpointDispatcher.EndpointAddress.ToString(),
                            channelDispatcher.State.ToString());
                    Console.WriteLine(sb.ToString());
                }
            }

            // The service runtime can now be accessed...
            Console.WriteLine("Press <ENTER> to terminate Receiver Proxy service");
            Console.WriteLine();
            Console.ReadLine();

            // Close the service runtime
            serviceHost.Close();
        }
    }

    class ReceiverProxyService : INonNormativePresto
    {
        public void submit1(string payload)
        {
            Console.WriteLine("Processing ReceiverProxyService(INonNormativePresto)::submit1");
            return;
        }

        public string submit2(string payload)
        {
            Console.WriteLine("Processing ReceiverProxyService(INonNormativePresto)::submit2");
            return payload;
        }
    }
}
